// -*-c++-*-
// Copyright (C) 1995 NEC Corporation, Tokyo, Japan.

#include "StringChange.h"
#include <InterViews/canvas.h>
#include <InterViews/color.h>
#include <InterViews/font.h>
#include <InterViews/hit.h>
#include <OS/string.h>
#include <stdio.h>
#include <string.h>

StringChange::StringChange(const String& s, const Font* f, const Color* c) : Glyph() {
    text_ = new CopyString(s);
    font_ = f;
    Resource::ref(font_);
    color_ = c;
    Resource::ref(color_);
    compute_metrics();
}

StringChange::StringChange(const char* text, const Font* f, const Color* c) : Glyph() {
    text_ = new CopyString(text);
    font_ = f;
    Resource::ref(font_);
    color_ = c;
    Resource::ref(color_);
    compute_metrics();
}

StringChange::StringChange(
    const char* text, int length, const Font* f, const Color* c
) : Glyph() {
    text_ = new CopyString(text, length);
    font_ = f;
    Resource::ref(font_);
    color_ = c;
    Resource::ref(color_);
    compute_metrics();
}

StringChange::~StringChange() {
    delete text_;
    Resource::unref(font_);
    Resource::unref(color_);
    delete char_widths_;
}

void StringChange::compute_metrics() {
    const Font* f = font_;
    const char* str = text_->string();
    int len = text_->length();
    FontBoundingBox b;
    f->string_bbox(str, len, b);
    ascent_ = b.font_ascent();
    descent_ = b.font_descent();
    left_ = b.left_bearing();
    right_ = b.right_bearing();
    width_ = b.width();
    char_widths_ = new Coord[len];
    for (int i = 0; i < len; i++) {
	char_widths_[i] = f->width(((u_char*)str)[i]);
    }
}

void StringChange::request(Requisition& requisition) const {
    Coord height = ascent_ + descent_;
    float alignment = (height == 0) ? 0 : descent_ / height;
    Requirement rx(width_, 0, 0, 0);
    Requirement ry(height, 0, 0, alignment);
    requisition.require(Dimension_X, rx);
    requisition.require(Dimension_Y, ry);
}

void StringChange::allocate(Canvas* c, const Allocation& a, Extension& ext) {
  canvas_ = c;
  allocation_ = a;
  extension_ = ext;
  Coord x = a.x();
  Coord y = a.y();
  ext.merge(canvas_, a);
}

void StringChange::draw(Canvas* c, const Allocation& a) const {
  Coord x = a.x();
  Coord y = a.y();
  const Font* f = font_;
  const Color* color = color_;
  const char* p = text_->string();
  const char* q = &p[text_->length()];
  Coord* cw = &char_widths_[0];
  for (; p < q; p++, cw++) {
    Coord width = *cw;
    c->character(f, *p, width, color, x, y);
    x += width;
  }
}

void StringChange::pick(Canvas*, const Allocation& a, int depth, Hit& h) {
    Coord x = h.left();
    if (h.right() >= a.left() && x < a.right() &&
	h.top() >= a.bottom() && h.bottom() < a.top()
    ) {
        int index = font_->index(
	    text_->string(), text_->length(), x - a.x(), true
	);
        h.target(depth, this, index);
    }
}

void StringChange::change_label(char* name) {
  text_ = new CopyString(name);
  set_label();
  Coord x = allocation_.x();
  Coord y = allocation_.y();
}

void StringChange::set_label(){
  undraw();
  compute_metrics();
  allocate(canvas_, allocation_, extension_);
  canvas_->damage(extension_.left(), extension_.bottom(),
		  extension_.right(), extension_.top());
  canvas_->damage_all();
}
