// -*-c++-*-
// Copyright (C) 1995 NEC Corporation, Tokyo, Japan.

#include "MultiMenu.h"
#include "MultiPlayer.h"

#include <Hs/HsSession.h>

#include <InterViews/action.h>
#include <InterViews/bitmap.h>
#include <InterViews/display.h>
#include <InterViews/event.h>
#include <InterViews/font.h>
#include <InterViews/input.h>
#include <InterViews/layout.h>
#include <InterViews/telltale.h>
#include <InterViews/page.h>
#include <InterViews/patch.h>
#include <InterViews/polyglyph.h>
#include <InterViews/rule.h>
#include <InterViews/session.h>
#include <InterViews/stencil.h>
#include <InterViews/style.h>

#include <IV-look/button.h>
#include <IV-look/dialogs.h>
#include <IV-look/fchooser.h>
#include <IV-look/field.h>
#include <IV-look/kit.h>

#include <IV-X11/xdisplay.h>

#include <OS/string.h>
#include <stdlib.h>
#include <string.h>

#define MBOXLEN 20
#define MBOXFONT "*-courier-medium-r-normal--18-180-75-75-*"

extern DataType type_check( const char* name );

declareActionCallback(MultiMenu)
implementActionCallback(MultiMenu)

declareFieldEditorCallback(MultiMenu)
implementFieldEditorCallback(MultiMenu)

declareFileChooserCallback(MultiMenu)
implementFileChooserCallback(MultiMenu)

MultiMenu::MultiMenu():Glyph()
{
  select_name_ = nil;
  body_ = make();
  window_ = nil;
  Resource::ref(body_);
}

void MultiMenu::request(Requisition& requisition) const
{
  if(body_ != nil){
     body_->request(requisition);
  }
}

void MultiMenu::allocate(Canvas* c, const Allocation& a, Extension& ext)
{
  if(body_ != nil){
    body_->allocate(c, a, ext);
  }
}

void MultiMenu::draw(Canvas* canvas, const Allocation& a) const
{
  if(body_ != nil)
    body_->draw(canvas, a);
}

void MultiMenu::pick(Canvas* c, const Allocation& a, int depth, Hit& h)
{
   if (body_ != nil) {
      body_->pick(c, a, depth, h);
   } else {
      Glyph::pick(c, a, depth, h);
   }        
}

Glyph* MultiMenu::make()
{
   WidgetKit& kit    = *WidgetKit::instance();
   LayoutKit& layout = *LayoutKit::instance();
   DialogKit& dialog = *DialogKit::instance();

   Action* act1 =
      new ActionCallback(MultiMenu)(this, &MultiMenu::pmap);
   Action* act2 =
      new ActionCallback(MultiMenu)(this, &MultiMenu::quit);

   Style* s;
   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute( "*PushButton*minimumWidth", "2" );
   s->attribute( "font", "*-helvetica-medium-r-normal--14-140-75-75-*" );
   char* path = getenv ("DATA_DIR");
   if(path)
       fchooser_ = 
          new FileChooser(path, &kit, kit.style(), 
		     new FileChooserCallback(MultiMenu)(this,
							&MultiMenu::send_path));
   else
       fchooser_ = 
          new FileChooser(".", &kit, kit.style(), 
		     new FileChooserCallback(MultiMenu)(this,
							&MultiMenu::send_path));
   kit.pop_style();
   fchooser_->dismiss(false);
   
   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute("*PushButton*minimumWidth", "2");
   s->attribute( "font", "*-helvetica-medium-r-normal--14-140-75-75-*" );
   Glyph* button1 = kit.push_button( "Play", act1 );
   Glyph* button2 = kit.push_button( "Quit", act2 );
   Glyph* bg = 
      kit.outset_frame(
	    layout.vmargin(
		   layout.hbox(layout.hglue(2),button1, 
			       layout.hglue(2),button2,layout.hglue(2)), 3 
			   ) );

   kit.pop_style();
   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute( "font", MBOXFONT );
   mbox_ = layout.vbox();
   mbox_->append( layout.hspace(kit.font()->width(' ')*20) );
   mbox_->append( layout.vglue() );
   patch_ = new Patch( mbox_ );
   kit.pop_style();

   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute( "font", "*-helvetica-medium-r-normal--14-140-75-75-*" );
   Glyph* mlabel = kit.label("Messages:");
   kit.pop_style();

   Glyph* mbody = 
      kit.outset_frame(
	     layout.margin(
		  layout.vbox( layout.vspace(2),
			       mlabel,
			       layout.vspace(2),
			       patch_ ), 2 ) );

   Glyph* body = 
      layout.hbox(
	 layout.vbox(
	 kit.outset_frame(
		   layout.margin( 
			kit.inset_frame( fchooser_ ), 5
				  )
			  )
		     ),
	 layout.vbox(
	 kit.outset_frame(
	           layout.margin( 
	                kit.inset_frame( mbody ), 5 
				  )
			  ),
	 kit.outset_frame(
	           layout.margin( 
	                kit.inset_frame( bg ), 5 
				  )
			  )
		     )
		  );
   
   return body;
}

void MultiMenu::map(){

   Window* w = window_;
   window_ = new ApplicationWindow( body_ );   
   window_->map();
   while( 2 < mbox_->count() )
      mbox_->remove(0);
   patch_->reallocate();
   patch_->redraw();
   
   Display* d = HsSession::instance()->default_display();

   if( select_name_ )
     delete[] select_name_;
   select_name_ = nil;
   if(w)
     delete w;
}

void MultiMenu::pmap()
{
  char buff[168];
  char name1[40], name2[40];
  int w,h,d;
  if(!select_name_){
    message(3);
    return;
  }
  

  FILE* fp;
  if( ( fp = fopen( select_name_,"r" ) ) == NULL )
     return;

  Display* dpy = HsSession::instance()->default_display();

  if( type_ == AVFILE && dpy->rep()->default_visual_->depth() == 24 )
    while( fgets( buff, 160, fp ) ){
      sscanf(buff,"%s %s %d %d %d",name1, name2, &w, &h, &d );
      if(!strcmp( "DigitizedVideo", name1 )){
	message(2);
	return;
      }
    }

  fclose(fp);

  if( type_ == NON )
    message(3);
  else if( ( dpy->rep()->default_visual_->depth() != 8 ) && 
	   ( type_ == MONO || type_ == COLOR ) )	 
     message(2);
  else{
     MultiPlayer* player = 
        new MultiPlayer( type_, select_name_, (MultiMenu *)this );
     player->map();
     window_->unmap();
  }
}

void MultiMenu::quit()
{
  Session::instance()->quit();
}

void MultiMenu::message( unsigned int flag )
{
  char messages[256];
  char buff[24];
  WidgetKit& kit    = *WidgetKit::instance();

  while( 2 < mbox_->count() )
    mbox_->remove(0);

  if( type_ ==  AVFILE )
    strcpy( buff, "AVFile" );
  else if( type_ ==  COLOR )
    strcpy( buff, "COLOR" );
  else if( type_ ==  MONO )
    strcpy( buff, "MONO" );
  else if( type_ ==  MPEG )
    strcpy( buff, "MPEG" );
  else if( type_ ==  JPEG )
    strcpy( buff, "JPEG" );

  switch( flag ){
  case 1:
    sprintf(messages, "I assume that %s is %s format.", select_name_, buff );
    break;
  case 2:
    sprintf(messages, "This data, %s is not a full color data.",
	    select_name_ );
    break;
  case 3:
    sprintf(messages, "Please select data before playing.");
    break;
  default:
    return;
  };

  char* cp = messages;
  int len = strlen(messages);

  kit.push_style( new Style( kit.style() ) );

  kit.style()->attribute( "font", MBOXFONT );

  while( 0 < len ){
    strncpy( buff, cp, MBOXLEN );
    if( MBOXLEN < len ) buff[MBOXLEN]='\0'; else buff[len]='\0';
    len -= MBOXLEN;
    mbox_->insert( mbox_->count()-2, kit.label(buff) );
    cp += MBOXLEN;
  }
  kit.pop_style();
  
  patch_->reallocate();
  patch_->redraw();
}

void MultiMenu::send_path( FileChooser* fchooser, boolean flag ){
  
  if(!flag)
    return;

  if( !fchooser->selected() ){
    fprintf( stderr, "bad name called\n");
    return;
  }
  
  select_name_ = 
    new char[ strlen( fchooser->selected()->string() ) + 1 ];
  strcpy( select_name_, fchooser->selected()->string() );

  type_ = type_check(select_name_);
  if( type_ == NON ){
    select_name_ = nil;
    return;
  }

  if( type_ != NON )
     message(1);
}

Window* MultiMenu::window(){ return window_; }

