// -*-c++-*-
// Copyright (C) 1995 NEC Corporation, Tokyo, Japan.

#include "BoundedValue.h"
#include "MultiPlayer.h"

#include <Hs/HsAVdesc.h>
#include <Hs/HsDigitizedVideo.h>
#include <Hs/HsMPEGVideo.h>
#include <Hs/HsJPEGVideo.h>
#include <Hs/HsViewer.h>
#include <Hs/HsDefaultColormap.h>

#include <InterViews/action.h>
#include <InterViews/bitmap.h>
#include <InterViews/display.h>
#include <InterViews/font.h>
#include <InterViews/input.h>
#include <InterViews/layout.h>
#include <InterViews/telltale.h>
#include <InterViews/page.h>
#include <InterViews/patch.h>
#include <InterViews/polyglyph.h>
#include <InterViews/rule.h>
#include <InterViews/session.h>
#include <InterViews/stencil.h>
#include <InterViews/style.h>

#include <IV-look/fchooser.h>
#include <IV-look/kit.h>
#include <IV-look/dialogs.h>
#include <IV-look/button.h>

#include <IV-X11/xdisplay.h>

#include <OS/string.h>
#include <stdlib.h>
#include <string.h>

#include "arrow.bm"

extern boolean _nolabel;
extern boolean _nospeed;

declareActionCallback(MultiPlayer)
implementActionCallback(MultiPlayer)

MultiPlayer::MultiPlayer(DataType t, const char* name, MultiMenu* mm ):Glyph(), mmenu_(mm), window_(nil), type_(t), bvalue_(nil), best_color_(false)
{
   HsMedia* m;
   speed_ = 1;
   filename_ = new char[ strlen(name) ];
   strcpy( filename_, name );

   switch(type_){
   case AVFILE:
      m = HsAVdesc::open(name);
      break;
   case COLOR:
      m = new HsDigitizedVideo(name);
      break;
   case MONO:
      m = new HsDigitizedVideo(name);
      break;
   case MPEG:
      m = new HsMPEGVideo(name);
      break;
   case JPEG:
      m = new HsJPEGVideo(name);
      break;
    case NON:
      break;
   }
   viewer_ = new HsViewer( m );

   if( !_nospeed ){
      bvalue_ = new BoundedValue( -3, 3 );
      bvalue_->scroll_to( Dimension_Y, 1 );
   }else
      bvalue_ = nil;

   body_ = make();

   Resource::ref(body_);
   if(mmenu_)
     window_ = new TransientWindow( body_ );
   else
     window_ = new ApplicationWindow( body_ );
}

void MultiPlayer::request(Requisition& requisition) const
{
  if(body_ != nil){
     body_->request(requisition);
  }
}

void MultiPlayer::allocate(Canvas* c, const Allocation& a, Extension& ext)
{
  if(body_ != nil){
    body_->allocate(c, a, ext);
  }
}

void MultiPlayer::draw(Canvas* canvas, const Allocation& a) const
{
  if(body_ != nil)
    body_->draw(canvas, a);
}

void MultiPlayer::pick(Canvas* c, const Allocation& a, int depth, Hit& h)
{
   if (body_ != nil) {
      body_->pick(c, a, depth, h);
   } else {
      Glyph::pick(c, a, depth, h);
   }        
}

Glyph* MultiPlayer::make()
{
   WidgetKit& kit    = *WidgetKit::instance();
   LayoutKit& layout = *LayoutKit::instance();
   DialogKit& dialog = *DialogKit::instance();

   Glyph* box1 = layout.hbox();

   Style* s;
   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute( "font", "*-helvetica-bold-r-normal--8-*" );
   s->attribute("*PushButton*minimumWidth", "2");

   Action* act1 =
      new ActionCallback(MultiPlayer)(this, &MultiPlayer::quit);
   Glyph* button1 = kit.push_button("Quit", act1 );
   box1->append(layout.hglue());
   box1->append(kit.inset_frame(layout.margin(button1,2)));
   
   if(mmenu_){
     Action* act2 =
       new ActionCallback(MultiPlayer)(this, &MultiPlayer::mmap);
     Glyph* button2 = kit.push_button("Menu", act2 );
     box1->append(layout.hglue());
     box1->append(kit.inset_frame(layout.margin(button2,2)));
   }
   
   Action* act3 =
      new ActionCallback(MultiPlayer)(this, &MultiPlayer::remap);
   Glyph* button3 = kit.push_button("Opt", act3 );
   box1->append(layout.hglue());
   box1->append(kit.inset_frame(layout.margin(button3,2)));
   box1->append(layout.hglue());

   Action* act4 =
      new ActionCallback(MultiPlayer)(this, &MultiPlayer::nsmap);
   Glyph* button4 = kit.push_button("Def", act4 );
   box1->append(layout.hglue());
   box1->append(kit.inset_frame(layout.margin(button4,2)));
   box1->append(layout.hglue());

   Glyph* label = nil;

   if(!_nolabel){
	char* cfp = filename_;
	for( int l = 0; filename_[l] != '\0'; l++ )
	  if( filename_[l] == '/' && filename_[l+1] != '\0' )
	    cfp = &filename_[l+1];
      label = layout.hbox( layout.hglue(), kit.label( cfp ) );
   }
   kit.pop_style();

   if(bvalue_){
      do_exec_ =
	 new ActionCallback(MultiPlayer)(this, &MultiPlayer::execute);
      HsSession* session = (HsSession*)Session::instance();
      proc_ = session->add_workproc(do_exec_);
   }

   Glyph* g = layout.vbox();
   
   if(!_nospeed){
      Glyph* arrow = 
	 new Stencil(new Bitmap(arrow_bits, arrow_width, arrow_height),
		     kit.foreground());
      kit.push_style( new Style( kit.style() ) );
      kit.style()->attribute("foreground", "red");
      Glyph* zarrow = 
	 new Stencil(new Bitmap(arrow_bits, arrow_width, arrow_height),
		     kit.foreground());
      kit.pop_style();

      Glyph* check_boad = layout.vbox(15);
      Coord mtsize = 28.0;

      kit.style()->find_attribute("minimumThumbSize",mtsize);
      check_boad->append(layout.vspace(mtsize/2));
      check_boad->append(arrow);
      check_boad->append(layout.vglue());
      check_boad->append(arrow);
      check_boad->append(layout.vglue());
      check_boad->append(arrow);
      check_boad->append(layout.vglue());
      check_boad->append(zarrow);
      check_boad->append(layout.vglue());
      check_boad->append(arrow);
      check_boad->append(layout.vglue());
      check_boad->append(arrow);
      check_boad->append(layout.vglue());
      check_boad->append(arrow);
      check_boad->append(layout.vspace(mtsize/2));

      g->append( layout.hbox( layout.vbox( label, viewer_ ),
			      kit.outset_frame( check_boad ),
			      layout.vbox( kit.vslider(bvalue_) ) ) );
   }else{
      g->append( layout.hbox( layout.vbox( label, viewer_ ) ) );
   }

   g->append( new HsViewerGadget( viewer_ ) );
   g->append( layout.margin(box1,3) );
   return kit.outset_frame(g);
}

void MultiPlayer::map()
{
   window_->map();
   if( type_ != MONO )
      HsDefaultColormap::install( window_ );
}

void MultiPlayer::mmap()
{
   viewer_->stop();
   if( type_ == MPEG )
      ((HsMPEGVideo *)viewer_->media())->defaultcolor();
   if( type_ == JPEG )
      ((HsJPEGVideo *)viewer_->media())->defaultcolor();

   if( type_ != MONO )
      HsDefaultColormap::uninstall( window_ );

   window_->unmap();

   if(mmenu_)
      mmenu_->map();
}

void MultiPlayer::quit()
{
  viewer_->stop();
  Session::instance()->quit();
}

void MultiPlayer::remap()
{

  int depth =
     HsSession::instance()->default_display()->rep()->default_visual_->depth();
  
  if( depth != 8 )
    return;

  switch(type_){
  case MPEG:
    {
       HsMPEGVideo* mpeg = ((HsMPEGVideo*)viewer_->media());
       mpeg->bestcolor();
       break;
    }
  case JPEG:
    {
       HsJPEGVideo* jpeg = ((HsJPEGVideo*)viewer_->media());
       jpeg->bestcolor();
       break;
    }
  }

}

void MultiPlayer::nsmap()
{
   if( type_ == MPEG )
      ((HsMPEGVideo *)viewer_->media())->defaultcolor();
   if( type_ == JPEG )
      ((HsJPEGVideo *)viewer_->media())->defaultcolor();
}

void MultiPlayer::start()
{
   window_->map();
   if( type_ != MONO )
      HsDefaultColormap::install( window_ );
}

void MultiPlayer::execute()
{
   if(viewer_->state() != HS_PLAY)
      return;
   HsSpeed s = (HsSpeed)bvalue_->cur_lower(Dimension_Y);
   HsSpeed vs = viewer_->speed();

   if( s != speed_ ){
      HsTime current = viewer_->current();
      viewer_->pause();
      viewer_->speed(s);
      speed_ =  s;
      viewer_->jog(current);
      viewer_->resume();
   } else {
      bvalue_->scroll_to( Dimension_Y, vs );
      speed_ = vs;
   }
}
   
